/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2008. All
 * Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights: Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
//
//
package com.ibm.rational.test.lm.spi.driver.test;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import junit.framework.TestCase;

import com.ibm.ccl.soa.deploy.core.ExtendedAttribute;
import com.ibm.ccl.soa.deploy.core.HostingLink;
import com.ibm.ccl.soa.deploy.core.Topology;
import com.ibm.ccl.soa.deploy.core.Unit;
import com.ibm.ccl.soa.deploy.operation.RunState;
import com.ibm.ccl.soa.deploy.server.Server;
import com.ibm.ccl.soa.deploy.server.ServerUnit;
import com.ibm.ccl.soa.deploy.virtualization.VirtualImage;
import com.ibm.ccl.soa.deploy.virtualization.VirtualImageCollection;
import com.ibm.ccl.soa.deploy.virtualization.VirtualImageCollectionUnit;
import com.ibm.ccl.soa.deploy.virtualization.VirtualImageUnit;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml;
import com.ibm.rational.test.lm.spi.driver.test.util.Registry;
import com.ibm.rational.test.lm.spi.driver.test.util.TestTracer;
import com.ibm.rational.test.lm.spi.driver.test.util.TopologyUtils;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml.ProviderServerInfo;

public class Virtualization extends TestCase {

	private static String virtualizationHostname = "";
	private static String virtualizationPortNumber = "";

	private static final String SINGLE_SLASH = "/";
	private static final String COLON = ":";
	private static final String STAR = "*";

	private static final String virtualimageUrlPart = "/virtualimage/";
	private static final String virtualmachineUrlPart = "/virtualmachine/";
	private static final String vmmanagerUrlPart = "/vmmanager/";
	private static final String vmcollectionUrlPart = "/vmcollection/";
	private static final String vicollectionUrlPart = "/vicollection/";
	private static final String deployVMImageUrlPart = "deploy/vmimage/";
	private static final String deployVICollectionUrlPart = "deploy/vicollection/";

	private static String baseVirtualimageUrl;
	private static String baseVirtualmachineUrl;
	private static String baseVmmanagerUrl;
	private static String baseVmcollectionUrl;
	private static String baseVicollectionUrl;

	private static HashMap<String, String> providerToConfig = new HashMap<String, String>();

	private static boolean isInited = false;

	protected void setUp() throws Exception {
		assertNotNull(Registry.getRegistry().getVirtualizationProvider());
		if (!isInited) {
			assertTrue(initSPIData());
			isInited = true;
		}
	}

	private boolean initSPIData() {
		boolean success = false;
		providerToConfig = ProviderXml.getProviderDocs();
		success = providerToConfig.size() > 0;
		ProviderServerInfo serverInfo = ProviderXml
				.getProviderServerInfo(providerToConfig.get(Registry
						.getRegistry().getVirtualizationProvidePluginId()));
		virtualizationHostname = serverInfo.hostname;
		virtualizationPortNumber = serverInfo.port;
		baseVirtualimageUrl = SINGLE_SLASH + virtualizationHostname + COLON
				+ virtualizationPortNumber + virtualimageUrlPart;
		baseVirtualmachineUrl = SINGLE_SLASH + virtualizationHostname + COLON
				+ virtualizationPortNumber + virtualmachineUrlPart;
		baseVmmanagerUrl = SINGLE_SLASH + virtualizationHostname + COLON
				+ virtualizationPortNumber + vmmanagerUrlPart;
		baseVmcollectionUrl = SINGLE_SLASH + virtualizationHostname + COLON
				+ virtualizationPortNumber + vmcollectionUrlPart;
		baseVicollectionUrl = SINGLE_SLASH + virtualizationHostname + COLON
				+ virtualizationPortNumber + vicollectionUrlPart;
		return success;
	}

	/**
	 * test call of init() for virtualization provider
	 */
	public void testInit() {
		try {
			Registry.getRegistry().getVirtualizationProvider().init(
					providerToConfig.get(Registry.getRegistry()
							.getVirtualizationProvidePluginId()),
					new TestTracer());
		} catch (Exception e) {
			fail("Exception in Init(): " + e.toString());
		}
	}

	/**
	 * test get() of /virtualimage/* to retrieve a TDM model contains the
	 * details of all the virtual images the provider knows about.
	 */
	public void testGetAllVirtualImages() {
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVirtualimageUrl + "*"));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);
			Topology virtualImages = TopologyUtils
					.parseTopologyFromXml(response);
			assertNotNull(virtualImages);
			assertTrue(virtualImages.getUnits().get(0) instanceof VirtualImageUnit);
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test get() of /virtualmachine/* to retrieve a TDM model contains the
	 * details of all the virtual machines the provider knows about.
	 */
	public void testGetAllVirtualMachines() {
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVirtualmachineUrl + "*"));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);
			Topology virtualMachines = TopologyUtils
					.parseTopologyFromXml(response);
			assertNotNull(virtualMachines);
			assertTrue(virtualMachines.getUnits().get(0) instanceof ServerUnit);
			ServerUnit su = (ServerUnit) virtualMachines.getUnits().get(0);
			assertTrue(((Server) su.getCapabilities().get(0)).isVirtual());
			RunState runState = getRunState(su);
			assertNotNull(runState);
			assertTrue(!runState.equals(RunState.UNKNOWN_LITERAL));
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test get() of /virtualmachine/<ID> to retrieve a TDM model of the
	 * specific machine identified by ID.
	 */
	public void testGetVirtualMachine() {
		try {
			String[] vmIds = getAllVirtualMachineIds();
			assertTrue(vmIds.length > 0);
			String vmId = vmIds[0];

			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVirtualmachineUrl + vmId));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);
			Topology virtualMachine = TopologyUtils
					.parseTopologyFromXml(response);
			assertNotNull(virtualMachine);
			assertTrue(virtualMachine.getUnits().get(0) instanceof ServerUnit);
			ServerUnit serverUnit = (ServerUnit) virtualMachine.getUnits().get(
					0);
			RunState runState = getRunState(serverUnit);
			assertNotNull(runState);
			assertTrue(!runState.equals(RunState.UNKNOWN_LITERAL));
			assertTrue(serverUnit.getCapabilities().get(0) instanceof Server);
			Server server = (Server) serverUnit.getCapabilities().get(0);
			assertTrue(server.isVirtual());
			assertEquals(vmId, server.getSystemId());
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test post() of /vmmanager/deploy/vmimage/<ID> to deploy the specific
	 * virtual image identified by its ID and get the ID for the created virtual
	 * machine.
	 */
	public void testPostDeployVMImage() {
		try {
			String imageId = getVirtualImageId();

			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().post(
							new URI(baseVmmanagerUrl + deployVMImageUrlPart
									+ imageId), "");
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);

			// get id of created virtual machine from response
			Topology vmTopo = TopologyUtils.parseTopologyFromXml(response);
			assertTrue(vmTopo.getUnits() != null
					&& vmTopo.getUnits().size() > 0);
			assertTrue(vmTopo.getUnits().get(0) instanceof ServerUnit);
			ServerUnit serverUnit = (ServerUnit) vmTopo.getUnits().get(0);
			assertTrue(serverUnit.getCapabilities() != null
					&& serverUnit.getCapabilities().size() > 0);
			RunState runState = getRunState(serverUnit);
			assertNotNull(runState);
			assertTrue(!runState.equals(RunState.UNKNOWN_LITERAL));
			assertTrue(serverUnit.getCapabilities().get(0) instanceof Server);
			Server createdVM = (Server) serverUnit.getCapabilities().get(0);
			assertTrue(createdVM.isVirtual());
			String vmId = TopologyUtils.parseVirtualMachineId(createdVM);
			assertNotNull(vmId);
			assertTrue(vmId.length() > 0);
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test delete() of /virtualmachine/<ID> to undeploy the specific virtual
	 * machine identified by its ID
	 */
	public void testUnDeployVMMachine() {
		try {
			String[] vmIds = getAllVirtualMachineIds();
			assertTrue(vmIds.length > 0);
			String vmId = vmIds[0];
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().delete(
							new URI(baseVirtualmachineUrl + vmId));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);

			// Check the virtual machines is deleted
			String[] newVMIds = getAllVirtualMachineIds();
			for (String newVMId : newVMIds) {
				assertFalse(vmId.equals(newVMId));
			}
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test get() of /vmcollection/* to retrieve a TDM model of all machine
	 * collection.
	 */
	public void testGetVirtualMachineCollection() {
		try {

			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVmcollectionUrl + STAR));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);
			Topology vMCollection = TopologyUtils
					.parseTopologyFromXml(response);
			assertNotNull(vMCollection);
			List<?> units = vMCollection.getUnits();
			Unit vmColUnit = null;
			if (units != null && units.size() > 0) {
				for (Object obj : units) {
					Unit unit = (Unit) obj;
					if (unit
							.getExtendedAttribute("VirtualMachineCollectionUnit_ExternalId") != null) {
						vmColUnit = unit;
						break;
					}
				}
			}
			assertNotNull(vmColUnit);
			// Get all of virtual machine from collection unit
			List<?> allLinks = vmColUnit.getHostingLinks();
			for (Object link : allLinks) {
				Unit hostedUnit = ((HostingLink) link).getHosted();
				if (hostedUnit instanceof ServerUnit) {
					ServerUnit serverUnit = (ServerUnit) hostedUnit;
					assertNotNull(serverUnit);
				}
			}
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test get() of /vicollection/* to retrieve a TDM model of all machine
	 * collection.
	 */
	public void testGetVirtualImageCollection() {
		try {

			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVicollectionUrl + STAR));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);
			Topology viCollection = TopologyUtils
					.parseTopologyFromXml(response);
			assertNotNull(viCollection);
			assertTrue(viCollection.getUnits().get(0) instanceof VirtualImageCollectionUnit);
			VirtualImageCollectionUnit viColUnit = (VirtualImageCollectionUnit) viCollection
					.getUnits().get(0);
			assertNotNull(viColUnit);
			assertTrue(viColUnit.getCapabilities().get(0) instanceof VirtualImageCollection);
			VirtualImageCollection viCol = (VirtualImageCollection) viColUnit
					.getCapabilities().get(0);
			assertNotNull(viCol);
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test post() of /vmmanager/deploy/vicollection/<ID> to deploy the
	 * specific virtual image identified by its ID and get the ID for the
	 * created virtual machine.
	 */
	public void testPostDeployVMImageCollection() {
		try {
			String[] imageCollectionIds = getAllVirtualImageCollectionIds();
			assertTrue(imageCollectionIds.length > 0);

			String imageCollectionId = imageCollectionIds[0];
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().post(
							new URI(baseVmmanagerUrl
									+ deployVICollectionUrlPart
									+ imageCollectionId), "");
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);

			// get id of created virtual machine collection from response
			Topology vmTopo = TopologyUtils.parseTopologyFromXml(response);
			assertTrue(vmTopo.getUnits() != null
					&& vmTopo.getUnits().size() > 0);
			List<?> units = vmTopo.getUnits();
			Unit vmColUnit = null;
			if (units != null && units.size() > 0) {
				for (Object obj : units) {
					Unit unit = (Unit) obj;
					if (unit
							.getExtendedAttribute("VirtualMachineCollectionUnit_ExternalId") != null) {
						vmColUnit = unit;
						break;
					}
				}
			}
			assertNotNull(vmColUnit);

		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/**
	 * test delete() of /vmcollection/<ID> to undeploy an existed virtual
	 * machine collection
	 * 
	 */
	public void testUndeployVirtualMachineCollection() {
		try {
			String[] vmColIds = getAllVirtualMachineCollectionIds();
			assertTrue(vmColIds.length > 0);
			String vmColId = vmColIds[0];

			String[] vmIds = getVMIdsInCollection(vmColId);
			assertTrue(vmIds.length > 0);
			List<String> vmIdsList = Arrays.asList(vmIds);

			// URI: delete() /hostname:port/vmcollection/<ID>
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().delete(
							new URI(baseVmcollectionUrl + vmColId));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);

			// Check the undeployed virtual machine collection whether succeed
			// to be deleted.
			String[] newVMColIds = getAllVirtualMachineCollectionIds();
			for (String newVMColId : newVMColIds) {
				assertFalse(vmColId.equals(newVMColId));
			}

			// Check the virtual machines in the undeployed collection whether
			// succeed to be deleted too
			String[] newVMIds = getAllVirtualMachineIds();
			for (String newVMId : newVMIds) {
				assertFalse(vmIdsList.contains(newVMId));
			}
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	/*
	 * Get one of virtual image's id from all of virtual images @return the
	 * first of fetched virtual image's id or null if no image found.
	 */
	private String getVirtualImageId() {
		String virtualImageId = null;
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVirtualimageUrl + "*"));
			if (restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK) {
				String response = restResponse.getXMLResponse();
				if (response != null && response.length() > 0) {
					Topology allVirtualImages = TopologyUtils
							.parseTopologyFromXml(response);
					List<?> units = allVirtualImages.getUnits();
					if (units != null && units.size() > 0) {
						if (units.get(0) instanceof VirtualImageUnit) {
							VirtualImageUnit vmUnit = (VirtualImageUnit) units
									.get(0);
							if (vmUnit.getCapabilities().get(0) instanceof VirtualImage) {
								virtualImageId = TopologyUtils
										.parseVirtualImageId((VirtualImage) vmUnit
												.getCapabilities().get(0));
							}
						}
					}
				}
			}
		} catch (URISyntaxException e) {
			fail(e.toString());
		}
		return virtualImageId;
	}

	/*
	 * Get ids of all of virtual machines. @return ids of all of virtual
	 * machines or empty array if no virtual machine found.
	 */
	private String[] getAllVirtualMachineIds() {
		List<String> allVMIds = new ArrayList<String>();
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVirtualmachineUrl + "*"));
			if (restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK) {
				String response = restResponse.getXMLResponse();
				if (response != null && response.length() > 0) {
					Topology allVirtualMachines = TopologyUtils
							.parseTopologyFromXml(response);
					List<?> units = allVirtualMachines.getUnits();
					if (units != null && units.size() > 0) {
						for (Object obj : units) {
							if (obj instanceof ServerUnit) {
								ServerUnit serverUnit = (ServerUnit) obj;
								if (serverUnit.getCapabilities().get(0) instanceof Server) {
									String vmId = TopologyUtils
											.parseVirtualMachineId((Server) serverUnit
													.getCapabilities().get(0));
									allVMIds.add(vmId);
								}
							}
						}
					}
				}
			}
		} catch (URISyntaxException e) {
			fail(e.toString());
		}
		return allVMIds.toArray(new String[allVMIds.size()]);
	}

	/*
	 * Get ids of all of virtual machine collections.
	 * 
	 * @return ids of all virtual machine collection or empty array if no
	 * virtual machine collection found.
	 */
	private String[] getAllVirtualMachineCollectionIds() {
		List<String> allVMColIds = new ArrayList<String>();
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVmcollectionUrl + "*"));
			if (restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK) {
				String response = restResponse.getXMLResponse();
				if (response != null && response.length() > 0) {
					Topology allVMCollections = TopologyUtils
							.parseTopologyFromXml(response);
					List<?> units = allVMCollections.getUnits();
					if (units != null && units.size() > 0) {
						for (Object obj : units) {
							Unit unit = (Unit) obj;
							if (unit
									.getExtendedAttribute("VirtualMachineCollectionUnit_ExternalId") != null) {
								String vmColId = (String) unit
										.getExtendedAttribute(
												"VirtualMachineCollectionUnit_ExternalId")
										.getValue();
								allVMColIds.add(vmColId);
							}
						}
					}
				}
			}
		} catch (URISyntaxException e) {
			fail(e.toString());
		}
		return allVMColIds.toArray(new String[allVMColIds.size()]);
	}

	/*
	 * Get all of ids of virtual machines in the virtual machine collection by
	 * its given id.
	 * 
	 */
	private String[] getVMIdsInCollection(String vmColId) {
		List<String> vmIds = new ArrayList<String>();
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVmcollectionUrl + vmColId));
			assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
			String response = restResponse.getXMLResponse();
			assertNotNull(response);
			Topology vmCol = TopologyUtils.parseTopologyFromXml(response);
			List<?> units = vmCol.getUnits();
			Unit vmColUnit = null;
			if (units != null && units.size() > 0) {
				for (Object obj : units) {
					Unit unit = (Unit) obj;
					if (unit
							.getExtendedAttribute("VirtualMachineCollectionUnit_ExternalId") != null) {
						vmColUnit = unit;
						break;
					}
				}
			}
			assertNotNull(vmColUnit);
			// Get all of virtual machine from collection unit
			List<?> allLinks = vmColUnit.getHostingLinks();
			for (Object link : allLinks) {
				Unit hostedUnit = ((HostingLink) link).getHosted();
				if (hostedUnit instanceof ServerUnit) {
					ServerUnit serverUnit = (ServerUnit) hostedUnit;
					Server vm = (Server) serverUnit.getCapabilities().get(0);
					String vmExternalId = vm.getSystemId();
					if (vmExternalId != null && vmExternalId.length() > 0) {
						vmIds.add(vmExternalId);
					}
				}
			}
			return vmIds.toArray(new String[vmIds.size()]);
		} catch (URISyntaxException e) {
			fail(e.toString());
		}
		return vmIds.toArray(new String[vmIds.size()]);
	}

	/*
	 * Get ids of all of virtual image collections. @return ids of all of
	 * virtual image collection or empty array if no virtual image collection
	 * found.
	 */
	private String[] getAllVirtualImageCollectionIds() {
		List<String> allVIColIds = new ArrayList<String>();
		try {
			IRestResponse restResponse = Registry.getRegistry()
					.getVirtualizationProvider().get(
							new URI(baseVicollectionUrl + "*"));
			if (restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK) {
				String response = restResponse.getXMLResponse();
				if (response != null && response.length() > 0) {
					Topology allVirtualMachines = TopologyUtils
							.parseTopologyFromXml(response);
					List<?> units = allVirtualMachines.getUnits();
					if (units != null && units.size() > 0) {
						for (Object obj : units) {
							if (obj instanceof VirtualImageCollectionUnit) {
								VirtualImageCollectionUnit viColUnit = (VirtualImageCollectionUnit) obj;
								if (viColUnit.getCapabilities().get(0) instanceof VirtualImageCollection) {
									String viColId = ((VirtualImageCollection) viColUnit
											.getCapabilities().get(0))
											.getCollectionId();
									allVIColIds.add(viColId);
								}
							}
						}
					}
				}
			}
		} catch (URISyntaxException e) {
			fail(e.toString());
		}
		return allVIColIds.toArray(new String[allVIColIds.size()]);
	}

	private RunState getRunState(Unit unit) {
		ExtendedAttribute runStateEA = unit
				.getExtendedAttribute("CurrentRunState"); //$NON-NLS-N$ 
		if (runStateEA != null) {
			return RunState.getByName((String) runStateEA.getValue());
		} else
			return RunState.UNKNOWN_LITERAL;
	}
}
